@@ -1,10 +1,7 @@
-""" Non-negative matrix factorization.
-"""
-# Author: Vlad Niculae
-#         Lars Buitinck
-#         Mathieu Blondel <mathieu@mblondel.org>
-#         Tom Dupre la Tour
-# License: BSD 3 clause
+"""Non-negative matrix factorization."""
+
+# Authors: The scikit-learn developers
+# SPDX-License-Identifier: BSD-3-Clause
 
 import itertools
 import time
@@ -27,15 +24,16 @@ from ..base import (
 from ..exceptions import ConvergenceWarning
 from ..utils import check_array, check_random_state, gen_batches, metadata_routing
 from ..utils._param_validation import (
-    Hidden,
     Interval,
     StrOptions,
     validate_params,
 )
+from ..utils.deprecation import _deprecate_Xt_in_inverse_transform
 from ..utils.extmath import randomized_svd, safe_sparse_dot, squared_norm
 from ..utils.validation import (
     check_is_fitted,
     check_non_negative,
+    validate_data,
 )
 from ._cdnmf_fast import _update_cdnmf_fast
 
@@ -909,7 +907,7 @@ def non_negative_factorization(
     X,
     W=None,
     H=None,
-    n_components="warn",
+    n_components="auto",
     *,
     init=None,
     update_H=True,
@@ -932,22 +930,19 @@ def non_negative_factorization(
 
     The objective function is:
 
-        .. math::
+    .. math::
 
-            L(W, H) &= 0.5 * ||X - WH||_{loss}^2
+        L(W, H) &= 0.5 * ||X - WH||_{loss}^2
 
-            &+ alpha\\_W * l1\\_ratio * n\\_features * ||vec(W)||_1
+                &+ alpha\\_W * l1\\_ratio * n\\_features * ||vec(W)||_1
 
-            &+ alpha\\_H * l1\\_ratio * n\\_samples * ||vec(H)||_1
+                &+ alpha\\_H * l1\\_ratio * n\\_samples * ||vec(H)||_1
 
-            &+ 0.5 * alpha\\_W * (1 - l1\\_ratio) * n\\_features * ||W||_{Fro}^2
+                &+ 0.5 * alpha\\_W * (1 - l1\\_ratio) * n\\_features * ||W||_{Fro}^2
 
-            &+ 0.5 * alpha\\_H * (1 - l1\\_ratio) * n\\_samples * ||H||_{Fro}^2
-
-    Where:
-
-    :math:`||A||_{Fro}^2 = \\sum_{i,j} A_{ij}^2` (Frobenius norm)
+                &+ 0.5 * alpha\\_H * (1 - l1\\_ratio) * n\\_samples * ||H||_{Fro}^2,
 
+    where :math:`||A||_{Fro}^2 = \\sum_{i,j} A_{ij}^2` (Frobenius norm) and
     :math:`||vec(A)||_1 = \\sum_{i,j} abs(A_{ij})` (Elementwise L1 norm)
 
     The generic norm :math:`||X - WH||_{loss}^2` may represent
@@ -982,15 +977,17 @@ def non_negative_factorization(
         If `update_H=False`, it is used as a constant, to solve for W only.
         If `None`, uses the initialisation method specified in `init`.
 
-    n_components : int or {'auto'} or None, default=None
-        Number of components, if n_components is not set all features
-        are kept.
+    n_components : int or {'auto'} or None, default='auto'
+        Number of components. If `None`, all features are kept.
         If `n_components='auto'`, the number of components is automatically inferred
         from `W` or `H` shapes.
 
         .. versionchanged:: 1.4
             Added `'auto'` value.
 
+        .. versionchanged:: 1.6
+            Default value changed from `None` to `'auto'`.
+
     init : {'random', 'nndsvd', 'nndsvda', 'nndsvdar', 'custom'}, default=None
         Method used to initialize the procedure.
 
@@ -1139,16 +1136,15 @@ class _BaseNMF(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator,
     """Base class for NMF and MiniBatchNMF."""
 
     # This prevents ``set_split_inverse_transform`` to be generated for the
-    # non-standard ``W`` arg on ``inverse_transform``.
-    # TODO: remove when W is removed in v1.5 for inverse_transform
-    __metadata_request__inverse_transform = {"W": metadata_routing.UNUSED}
+    # non-standard ``Xt`` arg on ``inverse_transform``.
+    # TODO(1.7): remove when Xt is removed in v1.7 for inverse_transform
+    __metadata_request__inverse_transform = {"Xt": metadata_routing.UNUSED}
 
     _parameter_constraints: dict = {
         "n_components": [
             Interval(Integral, 1, None, closed="left"),
             None,
             StrOptions({"auto"}),
-            Hidden(StrOptions({"warn"})),
         ],
         "init": [
             StrOptions({"random", "nndsvd", "nndsvda", "nndsvdar", "custom"}),
@@ -1169,7 +1165,7 @@ class _BaseNMF(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator,
 
     def __init__(
         self,
-        n_components="warn",
+        n_components="auto",
         *,
         init=None,
         beta_loss="frobenius",
@@ -1195,16 +1191,6 @@ class _BaseNMF(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator,
     def _check_params(self, X):
         # n_components
         self._n_components = self.n_components
-        if self.n_components == "warn":
-            warnings.warn(
-                (
-                    "The default value of `n_components` will change from `None` to"
-                    " `'auto'` in 1.6. Set the value of `n_components` to `None`"
-                    " explicitly to suppress the warning."
-                ),
-                FutureWarning,
-            )
-            self._n_components = None  # Keeping the old default value
         if self._n_components is None:
             self._n_components = X.shape[1]
 
@@ -1310,55 +1296,44 @@ class _BaseNMF(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator,
         self.fit_transform(X, **params)
         return self
 
-    def inverse_transform(self, Xt=None, W=None):
+    def inverse_transform(self, X=None, *, Xt=None):
         """Transform data back to its original space.
 
         .. versionadded:: 0.18
 
         Parameters
         ----------
+        X : {ndarray, sparse matrix} of shape (n_samples, n_components)
+            Transformed data matrix.
+
         Xt : {ndarray, sparse matrix} of shape (n_samples, n_components)
             Transformed data matrix.
 
-        W : deprecated
-            Use `Xt` instead.
-
-            .. deprecated:: 1.3
+            .. deprecated:: 1.5
+                `Xt` was deprecated in 1.5 and will be removed in 1.7. Use `X` instead.
 
         Returns
         -------
         X : ndarray of shape (n_samples, n_features)
             Returns a data matrix of the original shape.
         """
-        if Xt is None and W is None:
-            raise TypeError("Missing required positional argument: Xt")
 
-        if W is not None and Xt is not None:
-            raise ValueError("Please provide only `Xt`, and not `W`.")
-
-        if W is not None:
-            warnings.warn(
-                (
-                    "Input argument `W` was renamed to `Xt` in v1.3 and will be removed"
-                    " in v1.5."
-                ),
-                FutureWarning,
-            )
-            Xt = W
+        X = _deprecate_Xt_in_inverse_transform(X, Xt)
 
         check_is_fitted(self)
-        return Xt @ self.components_
+        return X @ self.components_
 
     @property
     def _n_features_out(self):
         """Number of transformed output features."""
         return self.components_.shape[0]
 
-    def _more_tags(self):
-        return {
-            "requires_positive_X": True,
-            "preserves_dtype": [np.float64, np.float32],
-        }
+    def __sklearn_tags__(self):
+        tags = super().__sklearn_tags__()
+        tags.input_tags.positive_only = True
+        tags.input_tags.sparse = True
+        tags.transformer_tags.preserves_dtype = ["float64", "float32"]
+        return tags
 
 
 class NMF(_BaseNMF):
@@ -1370,23 +1345,20 @@ class NMF(_BaseNMF):
 
     The objective function is:
 
-        .. math::
+    .. math::
 
-            L(W, H) &= 0.5 * ||X - WH||_{loss}^2
+        L(W, H) &= 0.5 * ||X - WH||_{loss}^2
 
-            &+ alpha\\_W * l1\\_ratio * n\\_features * ||vec(W)||_1
+                &+ alpha\\_W * l1\\_ratio * n\\_features * ||vec(W)||_1
 
-            &+ alpha\\_H * l1\\_ratio * n\\_samples * ||vec(H)||_1
+                &+ alpha\\_H * l1\\_ratio * n\\_samples * ||vec(H)||_1
 
-            &+ 0.5 * alpha\\_W * (1 - l1\\_ratio) * n\\_features * ||W||_{Fro}^2
+                &+ 0.5 * alpha\\_W * (1 - l1\\_ratio) * n\\_features * ||W||_{Fro}^2
 
-            &+ 0.5 * alpha\\_H * (1 - l1\\_ratio) * n\\_samples * ||H||_{Fro}^2
+                &+ 0.5 * alpha\\_H * (1 - l1\\_ratio) * n\\_samples * ||H||_{Fro}^2,
 
-    Where:
-
-    :math:`||A||_{Fro}^2 = \\sum_{i,j} A_{ij}^2` (Frobenius norm)
-
-    :math:`||vec(A)||_1 = \\sum_{i,j} abs(A_{ij})` (Elementwise L1 norm)
+    where :math:`||A||_{Fro}^2 = \\sum_{i,j} A_{ij}^2` (Frobenius norm) and
+    :math:`||vec(A)||_1 = \\sum_{i,j} abs(A_{ij})` (Elementwise L1 norm).
 
     The generic norm :math:`||X - WH||_{loss}` may represent
     the Frobenius norm or another supported beta-divergence loss.
@@ -1407,15 +1379,17 @@ class NMF(_BaseNMF):
 
     Parameters
     ----------
-    n_components : int or {'auto'} or None, default=None
-        Number of components, if n_components is not set all features
-        are kept.
+    n_components : int or {'auto'} or None, default='auto'
+        Number of components. If `None`, all features are kept.
         If `n_components='auto'`, the number of components is automatically inferred
         from W or H shapes.
 
         .. versionchanged:: 1.4
             Added `'auto'` value.
 
+        .. versionchanged:: 1.6
+            Default value changed from `None` to `'auto'`.
+
     init : {'random', 'nndsvd', 'nndsvda', 'nndsvdar', 'custom'}, default=None
         Method used to initialize the procedure.
         Valid options:
@@ -1577,7 +1551,7 @@ class NMF(_BaseNMF):
 
     def __init__(
         self,
-        n_components="warn",
+        n_components="auto",
         *,
         init=None,
         solver="cd",
@@ -1658,8 +1632,8 @@ class NMF(_BaseNMF):
         W : ndarray of shape (n_samples, n_components)
             Transformed data.
         """
-        X = self._validate_data(
-            X, accept_sparse=("csr", "csc"), dtype=[np.float64, np.float32]
+        X = validate_data(
+            self, X, accept_sparse=("csr", "csc"), dtype=[np.float64, np.float32]
         )
 
         with config_context(assume_finite=True):
@@ -1714,8 +1688,6 @@ class NMF(_BaseNMF):
         n_iter_ : int
             Actual number of iterations.
         """
-        check_non_negative(X, "NMF (input X)")
-
         # check parameters
         self._check_params(X)
 
@@ -1769,8 +1741,7 @@ class NMF(_BaseNMF):
         if n_iter == self.max_iter and self.tol > 0:
             warnings.warn(
                 "Maximum number of iterations %d reached. Increase "
-                "it to improve convergence."
-                % self.max_iter,
+                "it to improve convergence." % self.max_iter,
                 ConvergenceWarning,
             )
 
@@ -1791,8 +1762,13 @@ class NMF(_BaseNMF):
             Transformed data.
         """
         check_is_fitted(self)
-        X = self._validate_data(
-            X, accept_sparse=("csr", "csc"), dtype=[np.float64, np.float32], reset=False
+        X = validate_data(
+            self,
+            X,
+            accept_sparse=("csr", "csc"),
+            dtype=[np.float64, np.float32],
+            reset=False,
+            ensure_non_negative=True,
         )
 
         with config_context(assume_finite=True):
@@ -1813,23 +1789,20 @@ class MiniBatchNMF(_BaseNMF):
 
     The objective function is:
 
-        .. math::
+    .. math::
 
-            L(W, H) &= 0.5 * ||X - WH||_{loss}^2
+        L(W, H) &= 0.5 * ||X - WH||_{loss}^2
 
-            &+ alpha\\_W * l1\\_ratio * n\\_features * ||vec(W)||_1
+                &+ alpha\\_W * l1\\_ratio * n\\_features * ||vec(W)||_1
 
-            &+ alpha\\_H * l1\\_ratio * n\\_samples * ||vec(H)||_1
+                &+ alpha\\_H * l1\\_ratio * n\\_samples * ||vec(H)||_1
 
-            &+ 0.5 * alpha\\_W * (1 - l1\\_ratio) * n\\_features * ||W||_{Fro}^2
+                &+ 0.5 * alpha\\_W * (1 - l1\\_ratio) * n\\_features * ||W||_{Fro}^2
 
-            &+ 0.5 * alpha\\_H * (1 - l1\\_ratio) * n\\_samples * ||H||_{Fro}^2
+                &+ 0.5 * alpha\\_H * (1 - l1\\_ratio) * n\\_samples * ||H||_{Fro}^2,
 
-    Where:
-
-    :math:`||A||_{Fro}^2 = \\sum_{i,j} A_{ij}^2` (Frobenius norm)
-
-    :math:`||vec(A)||_1 = \\sum_{i,j} abs(A_{ij})` (Elementwise L1 norm)
+    where :math:`||A||_{Fro}^2 = \\sum_{i,j} A_{ij}^2` (Frobenius norm) and
+    :math:`||vec(A)||_1 = \\sum_{i,j} abs(A_{ij})` (Elementwise L1 norm).
 
     The generic norm :math:`||X - WH||_{loss}^2` may represent
     the Frobenius norm or another supported beta-divergence loss.
@@ -1846,15 +1819,17 @@ class MiniBatchNMF(_BaseNMF):
 
     Parameters
     ----------
-    n_components : int or {'auto'} or None, default=None
-        Number of components, if `n_components` is not set all features
-        are kept.
+    n_components : int or {'auto'} or None, default='auto'
+        Number of components. If `None`, all features are kept.
         If `n_components='auto'`, the number of components is automatically inferred
         from W or H shapes.
 
         .. versionchanged:: 1.4
             Added `'auto'` value.
 
+        .. versionchanged:: 1.6
+            Default value changed from `None` to `'auto'`.
+
     init : {'random', 'nndsvd', 'nndsvda', 'nndsvdar', 'custom'}, default=None
         Method used to initialize the procedure.
         Valid options:
@@ -2018,7 +1993,7 @@ class MiniBatchNMF(_BaseNMF):
 
     def __init__(
         self,
-        n_components="warn",
+        n_components="auto",
         *,
         init=None,
         batch_size=1024,
@@ -2245,8 +2220,8 @@ class MiniBatchNMF(_BaseNMF):
         W : ndarray of shape (n_samples, n_components)
             Transformed data.
         """
-        X = self._validate_data(
-            X, accept_sparse=("csr", "csc"), dtype=[np.float64, np.float32]
+        X = validate_data(
+            self, X, accept_sparse=("csr", "csc"), dtype=[np.float64, np.float32]
         )
 
         with config_context(assume_finite=True):
@@ -2374,8 +2349,12 @@ class MiniBatchNMF(_BaseNMF):
             Transformed data.
         """
         check_is_fitted(self)
-        X = self._validate_data(
-            X, accept_sparse=("csr", "csc"), dtype=[np.float64, np.float32], reset=False
+        X = validate_data(
+            self,
+            X,
+            accept_sparse=("csr", "csc"),
+            dtype=[np.float64, np.float32],
+            reset=False,
         )
 
         W = self._solve_W(X, self.components_, self._transform_max_iter)
@@ -2416,7 +2395,8 @@ class MiniBatchNMF(_BaseNMF):
         """
         has_components = hasattr(self, "components_")
 
-        X = self._validate_data(
+        X = validate_data(
+            self,
             X,
             accept_sparse=("csr", "csc"),
             dtype=[np.float64, np.float32],
